#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

global cat
set cat(def) { \
	   {{SDSS Data Release 5} catsdss sdss DR5} \
	   {- - -} \
	   {{SAO Star Catalog J2000} catsao cds {I/131A}} \
	   {{HST Guide Star Catalog} cathst cds {I/254}} \
	   {{GSC 2.2 Catalog} catgsc cds {I/271}} \
	   {{Tycho-2 Catalog} cattycho cds {I/259}} \
	   {{Hipparcos AC 2000.2 Catalog} catac cds {I/275}} \
	   {{USNO-A2.0 Catalog} catua2 cds {I/252}} \
	   {{USNO-B1.0 Catalog} catub1 cds {I/284}} \
	   {{USNO UCAC2 Catalog} catucac2 cds {I/289}} \
	   {- - -} \
	   {{2MASS All-Sky Catalog of Point Sources} cat2mass cds {II/246}}\
	   {{DENIS Database} catdenis cds {B/denis}} \
	   {- - -} \
	   {{Second ROSAT PSPC Catalog} catrosat cds {IX/30}} \
	   {{ROSAT All-Sky Survey Faint Source Catalog} catrosf cds {IX/29}}\
	   {- - -} \
	   {{FIRST Survey Catalog} catfirst cds {VIII/71}} \
}

proc CATApply {varname sync} {
    upvar #0 $varname var
    global $varname
    global menu

    global debug
    if {$debug(tcl,cat)} {
	puts "CATApply $varname $sync"
    }

    if {$var(catalog) == {}} {
	Error "No Catalog specified"
	return
    }

    set var(sync) $sync
    ARApply $varname
    if {$var(name) != {}} {
	set var(system) wcs
	set var(sky) fk5
	ARCoord $varname

	NSVRServer $varname
    } else {
	CATServer $varname
    }
}

proc CATDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global $var(tbldb)
    global $var(dir)
    global $var(symdb)
    global $var(symdl)
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATDestroy $varname"
    }

    upvar #0 $var(symdl) svar
    if [info exists svar(top)] {
	if [winfo exists $svar(top)] {
	    CATSymDestroy $var(symdl)
	}
    }

    catch {unset $var(symdb)}
    catch {unset $var(dir)}
    catch {unset $var(tbldb)}
    catch {unset $var(catdb)}

    set i [lsearch $cat(cats) $varname]
    if {$i>=0} {
	set cat(cats) [lreplace $cat(cats) $i $i]
    }

    ARDestroy $varname
}

proc CATDialog {varname format catalog title action} {
    upvar #0 $varname var
    global $varname

    global ds9
    global menu
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATDialog $varname :$format:$catalog:$title:$action"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # CAT variables
    lappend cat(cats) $varname

    # AR variables
    ARInit $varname CATServer

    # CAT variables
    set var(dir) ${varname}dir
    set var(catdb) ${varname}catdb
    set var(tbldb) ${varname}tbldb
    set var(symdb) ${varname}symdb
    set var(symdl) ${varname}symdl
    set var(fltdl) ${varname}fltdl

    set var(server) $cat(server)
    set var(system) $cat(system)
    set var(sky) $cat(sky)
    set var(skyformat) $cat(skyformat)
    set var(rformat) $cat(rformat)
    set var(width) $cat(width)
    set var(height) $cat(height)
    set var(max) $cat(max)
    set var(allrows) $cat(allrows)

    set var(blink) 0
    set var(blink,count) 0
    set var(blink,marker) {}

    CATSet $varname $format $catalog $title
    CATSymDBInit $varname

    # top
    set w $var(top)
    set t "Catalog Tool"

    # create the window
    toplevel $w -colormap $ds9(main)
    wm title $w $t
    wm iconname $w $t
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CATDestroy $varname"
    wm minsize $w 500 600

    # menu
    $w configure -menu $var(mb)
    menu $var(mb) -tearoff 0

    # file
    $var(mb) add cascade -label $menu(file) -menu $var(mb).file
    menu $var(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).file add cascade -label "Catalogs" \
	-menu $var(mb).file.cat
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,load) \
	-command "CATLoadFile $varname"
    $var(mb).file add cascade -label $menu(file,loadother) \
	-menu $var(mb).file.other
    $var(mb).file add command -label $menu(file,save) \
	-command "CATSaveFile $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "Display Catalog Header..." \
	-command "CATHeader $varname"
    $var(mb).file add separator
    $var(mb).file add command -label Filter \
	-command "CATTable $varname"
    $var(mb).file add command -label $menu(file,clear) \
	-command "CATOff $varname"
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,retrieve) \
	-command "CATApply $varname 0"
    $var(mb).file add command -label $menu(file,cancel) \
	-command "ARCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label Acknowledgement \
	-command "CATAck $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "Update from Current Frame" \
	-command "CATUpdate $varname"
    $var(mb).file add command -label "Update from Current Crosshair" \
	-command "CATCrosshair $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "Copy to Regions" \
	-command "CATPlotRegions $varname"
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,print) \
	-command "CATPrint $varname"
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,close) \
	-command "CATDestroy $varname"

    menu $var(mb).file.cat -tearoff 0 -selectcolor $menu(selectcolor)
    CATCatalogMenu $varname

    menu $var(mb).file.other -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).file.other add command -label "FITS" -state disable
    $var(mb).file.other add command -label "CSV" \
	-command "CATLoadCSVFile $varname"

    # edit
    AREditMenu $varname

    # catalog
    $var(mb) add cascade -label "Catalog Server" -menu $var(mb).server
    menu $var(mb).server -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).server add radiobutton -label "CDS" \
	-variable ${varname}(server) -value cds
    $var(mb).server add radiobutton -label "SAO" \
	-variable ${varname}(server) -value sao
    $var(mb).server add radiobutton -label "CADC" \
	-variable ${varname}(server) -value cadc
    $var(mb).server add radiobutton -label "ADAC" \
	-variable ${varname}(server) -value adac
    $var(mb).server add radiobutton -label "IUCAA" \
	-variable ${varname}(server) -value iucaa
    $var(mb).server add radiobutton -label "BEJING" \
	-variable ${varname}(server) -value bejing
    $var(mb).server add radiobutton -label "CAMBRIDGE UK" \
	-variable ${varname}(server) -value cambridge
    $var(mb).server add radiobutton -label "UKIRT HAWAII" \
	-variable ${varname}(server) -value ukirt

    # name server
    NSVRServerMenu $varname

    # coordinate
    $var(mb) add cascade -label Coord -menu $var(mb).coord
    menu $var(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).coord add radiobutton -label $menu(wcs,degrees) \
	-variable ${varname}(skyformat) -value degrees \
	-command "CATCoord $varname"
    $var(mb).coord add radiobutton -label $menu(wcs,sexagesimal) \
	-variable ${varname}(skyformat) -value sexagesimal \
	-command "CATCoord $varname"

    # size
    $var(mb) add cascade -label Size -menu $var(mb).size
    menu $var(mb).size -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).size add radiobutton -label $menu(wcs,degrees) \
	-variable ${varname}(rformat) -value degrees \
	-command "CATCoord $varname"
    $var(mb).size add radiobutton -label $menu(wcs,arcmin) \
	-variable ${varname}(rformat) -value arcmin \
	-command "CATCoord $varname"
    $var(mb).size add radiobutton -label $menu(wcs,arcsec) \
	-variable ${varname}(rformat) -value arcsec \
	-command "CATCoord $varname"

    # symbol
    global $var(symdb)
    set flt $var(symdb)
    set sn [starbase_colnum $var(symdb) shape]
    set cn [starbase_colnum $var(symdb) color]

    $var(mb) add cascade -label Symbol -menu $var(mb).symbol
    menu $var(mb).symbol -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).symbol add radiobutton -label $menu(shape,point,circle) \
	-variable ${flt}(1,$sn) -value {circle point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,box) \
	-variable ${flt}(1,$sn) -value {box point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,diamond) \
	-variable ${flt}(1,$sn) -value {diamond point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,cross) \
	-variable ${flt}(1,$sn) -value {cross point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,x) \
	-variable ${flt}(1,$sn) -value {x point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,arrow) \
	-variable ${flt}(1,$sn) -value {arrow point} \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,point,boxcircle) \
	-variable ${flt}(1,$sn) -value {boxcircle point} \
	-command "CATPlot $varname"
    $var(mb).symbol add separator
    $var(mb).symbol add radiobutton -label $menu(shape,circle) \
	-variable ${flt}(1,$sn) -value circle \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,ellipse) \
	-variable ${flt}(1,$sn) -value ellipse \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,box) \
	-variable ${flt}(1,$sn) -value box \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(shape,text) \
	-variable ${flt}(1,$sn) -value text \
	-command "CATPlot $varname"
    $var(mb).symbol add separator
    $var(mb).symbol add radiobutton -label $menu(color,white) \
	-variable ${flt}(1,$cn) -value white \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,black) \
	-variable ${flt}(1,$cn) -value black \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,red) \
	-variable ${flt}(1,$cn) -value red \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,green) \
	-variable ${flt}(1,$cn) -value green \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,blue) \
	-variable ${flt}(1,$cn) -value blue \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,cyan) \
	-variable ${flt}(1,$cn) -value cyan \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,magenta) \
	-variable ${flt}(1,$cn) -value magenta \
	-command "CATPlot $varname"
    $var(mb).symbol add radiobutton -label $menu(color,yellow) \
	-variable ${flt}(1,$cn) -value yellow \
	-command "CATPlot $varname"
    $var(mb).symbol add separator
    $var(mb).symbol add command -label "Advanced..." \
	-command "CATSymDialog $varname"

    # dialog
    frame $w.param -relief groove -borderwidth 2
    frame $w.tbl -relief groove -borderwidth 2
    frame $w.status -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x  -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.f
    pack $w.param.f -side left

    label $w.param.f.ttitle -text "Catalog"
    label $w.param.f.title -textvariable ${varname}(title) \
	-relief groove -borderwidth 2 -width 50 -anchor w

    label $w.param.f.tcat -text "Id"
    label $w.param.f.cat -textvariable ${varname}(catalog) \
	-relief groove -borderwidth 2 -width 40 -anchor w

    label $w.param.f.nametitle -text "Name"
    entry $w.param.f.name -textvariable ${varname}(name) -width 40

    set var(xname) [label $w.param.f.xtitle -text "" \
			-font {symbol 12} -width 1]
    entry $w.param.f.x -textvariable ${varname}(x) -width 14
    set var(yname) [label $w.param.f.ytitle -text "" \
			-font {symbol 12} -width 1]
    entry $w.param.f.y -textvariable ${varname}(y) -width 14
    label $w.param.f.system -textvariable ${varname}(sky) \
	-width 10 -relief groove

    label $w.param.f.wtitle -text "Width"
    entry $w.param.f.w -textvariable ${varname}(width) -width 14
    label $w.param.f.htitle -text "Height"
    entry $w.param.f.h -textvariable ${varname}(height) -width 14
    label $w.param.f.format -textvariable ${varname}(rformat) \
	-width 10 -relief groove

    label $w.param.f.mfilter -text "Filter"
    entry $w.param.f.filter -textvariable ${varname}(filter) -width 40
    button $w.param.f.bfilter -text "Edit" \
	-command "CATEditDialog $varname filter $var(catdb)"

    label $w.param.f.msort -text "Sort"
    menubutton $w.param.f.sort -textvariable ${varname}(sort) \
	-menu $w.param.f.sort.m -relief raised -bd 2 -width 14
    radiobutton $w.param.f.isort -text "Incr" \
	-variable ${varname}(sort,dir) -value "-increasing" \
	-selectcolor $menu(selectcolor) -command "CATTable $varname"
    radiobutton $w.param.f.dsort -text "Decr" \
	-variable ${varname}(sort,dir) -value "-decreasing" \
	-selectcolor $menu(selectcolor) -command "CATTable $varname"

    label $w.param.f.mtitle -text "Max Rows"
    entry $w.param.f.max -textvariable ${varname}(max) -width 14

    checkbutton $w.param.f.all -selectcolor $menu(selectcolor) \
	-text "All Rows" -variable ${varname}(allrows)

    label $w.param.f.ftitle -text "Found" 
    set var(found) [label $w.param.f.found  \
			-relief groove -borderwidth 2 -width 14 -anchor w]

    label $w.param.f.mra -text "RA"
    menubutton $w.param.f.ra -textvariable ${varname}(ra) \
	-menu $w.param.f.ra.m -relief raised -bd 2 -width 14
    label $w.param.f.mdec -text "DEC"
    menubutton $w.param.f.dec -textvariable ${varname}(dec) \
	-menu $w.param.f.dec.m -relief raised -bd 2 -width 14

    grid rowconfigure $w.param.f 0 -pad 4
    grid rowconfigure $w.param.f 1 -pad 4
    grid rowconfigure $w.param.f 2 -pad 4
    grid rowconfigure $w.param.f 3 -pad 4
    grid rowconfigure $w.param.f 4 -pad 4
    grid rowconfigure $w.param.f 5 -pad 4
    grid rowconfigure $w.param.f 6 -pad 4
    grid rowconfigure $w.param.f 7 -pad 4

    grid $w.param.f.ttitle $w.param.f.title - - - -padx 4 -pady 1 -sticky w
    grid $w.param.f.tcat $w.param.f.cat - - -padx 4 -pady 1 -sticky w
    grid $w.param.f.nametitle $w.param.f.name - - -padx 4 -pady 1 -sticky w
    grid $w.param.f.xtitle $w.param.f.x $w.param.f.ytitle $w.param.f.y \
	$w.param.f.system -padx 4 -pady 1 -sticky w
    grid $w.param.f.wtitle $w.param.f.w $w.param.f.htitle \
	$w.param.f.h $w.param.f.format -padx 4 -pady 1 -sticky w
    grid $w.param.f.mfilter $w.param.f.filter - - $w.param.f.bfilter \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.msort $w.param.f.sort $w.param.f.isort $w.param.f.dsort \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.mtitle $w.param.f.max $w.param.f.ftitle $w.param.f.found \
	$w.param.f.all -padx 4 -pady 1 -sticky w
    grid $w.param.f.mra $w.param.f.ra $w.param.f.mdec $w.param.f.dec \
	-padx 4 -pady 1 -sticky w

    # table
    set var(tbl) [table $var(top).tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(tbldb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $cat(mincols) \
		      -rows $cat(minrows) \
		      -width -1 \
		      -height -1 \
		      -maxwidth 300 \
		      -maxheight 300 \
		      -titlerows 1 \
		      -xscrollcommand [list $var(top).tbl.xscroll set]\
		      -yscrollcommand [list $var(top).tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -browsecommand [list CATSelectCB $varname] \
		     ]

    scrollbar $var(top).tbl.yscroll -command [list $var(tbl) yview] \
	-orient vertical
    scrollbar $var(top).tbl.xscroll -command [list $var(tbl) xview] \
	-orient horizontal

    grid $var(tbl) $var(top).tbl.yscroll -sticky news
    grid $var(top).tbl.xscroll -stick news
    grid rowconfigure $var(top).tbl 0 -weight 1
    grid columnconfigure $var(top).tbl 0 -weight 1

    # status
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.item -anchor w -pady 4

    # buttons
    set var(apply) [button $w.buttons.apply -text $menu(file,retrieve) \
			-command "CATApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text $menu(file,cancel) \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.filter -text Filter -command "CATTable $varname"
    button $w.buttons.clear -text $menu(file,clear) \
	-command "CATOff $varname"
    button $w.buttons.close -text $menu(file,close) \
	-command "CATDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.filter \
	$w.buttons.clear $w.buttons.close -side left -padx 10 -expand true

    # needs to go after sort menu button is defined
    CATSortMenu $varname
    CATRADECMenu $varname
    CATServerMenu $varname

    CATCoord $varname
    ARStatus $varname {}

    switch -- $action {
	apply {CATApply $varname 0}
	sync {CATApply $varname 1}
	search {CATCDSSearch $varname}
	none {}
    }
}

# Dialog Commands

proc CATLoadFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn starbase_read
    }
}

proc CATLoadCSVFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catcsvfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn CATCSVRead
    }
}

proc CATLoadFn {varname fn reader} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoadFn $varname $fn $reader"
    }

    CATOff $varname
    CATSet $varname {} {} $fn

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(width) {}
    set var(height) {}

    if {[file exists $fn]} {
	$reader $var(catdb) $fn
    } else {
	Error "Unable to open catalog file $fn"
	return
    }

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

proc CATSaveFile {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    set fn [SaveFileDialog catfbox]
    if {$fn != {}} {
	starbase_write $var(tbldb) $fn
    }
}

proc CATOff {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global $var(tbldb)
    global current

    catch {unset $var(catdb)}
    catch {unset $var(tbldb)}
    set db $var(tbldb)
    set ${db}(Nrows) {}

    $var(tbl) selection clear all

    if {$current(frame) != {}} {
	$current(frame) marker catalog $varname delete
    }

    CATSortMenu $varname
    CATRADECMenu $varname
    set var(filter) {}
    set var(sort) {}
    set var(ra) {}
    set var(dec) {}
}

proc CATCoord {varname} {
    upvar #0 $varname var
    global $varname

    ARCoord $varname
    CATUpdate $varname
}

proc CATUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATUpdate $varname"
    }

    global current
    if {$current(frame) != {} } {
	if {[$current(frame) has wcs equatorial $var(system)]} {
	    set coord [$current(frame) get cursor \
			   $var(system) $var(sky) $var(skyformat)]
	    set var(x) [lindex $coord 0]
	    set var(y) [lindex $coord 1]

	    set size [$current(frame) get fits size \
			  $var(system) $var(sky) $var(rformat)]
	    set var(width) [lindex $size 0]
	    set var(height) [lindex $size 1]
	    set var(name) {}
	}
    }
}

proc CATCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    global current
    if {$current(frame) != {} } {
	if {[$current(frame) has wcs equatorial $var(system)]} {
	    set coord [$current(frame) get crosshair \
			   $var(system) $var(sky) sexagesimal]
	    set var(x) [lindex $coord 0]
	    set var(y) [lindex $coord 1]
	    set var(name) {}
	}
    }
}

proc CATAck {varname} {
    upvar #0 $varname var
    global $varname

    switch $var(format) {
	cds {CATCDSAck $varname}
	sdss {CATSDSSAck $varname}
    }
}

proc CATPrint {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global ps

    if {[PrintDialog txt]} { 
	if {$ps(dest) == "file"} {
	    catch {set ch [open "| cat > $ps(filename)" w]}
	} else {
	    catch {set ch [open "| $ps(cmd)" w]}
	}

	if {$ch != {}} {
	    starbase_writefp $var(tbldb) $ch
	    close $ch
	} else {
	    Error "An error as occurred while printing."
	    return
	}
    }
}

proc CATHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    SimpleTextDialog ${varname}hdr "$var(title) Header" \
	80 20 insert top [CATGetHeader $varname]
}

proc CATGetHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if [CATValidDB $var(tbldb)] {
	set hdr {}
	set nl [expr $${var(tbldb)}(HLines) - 2]
	for {set ll 1} {$ll <= $nl} {incr ll} {
	    append hdr "[expr $${var(tbldb)}(H_$ll)]\n"
	}
	return $hdr
    }
    return {}
}

# Other procedures

proc CATServer {varname} {
    upvar #0 $varname var
    global $varname
    global menu

    global debug
    if {$debug(tcl,cat)} {
	puts "CATServer $varname"
    }

    if {($var(x) != {}) && 
	($var(y) != {}) && 
	($var(width) != {}) && 
	($var(height) != {})} {

	ARStatus $varname "Searching [string range $var(title) 0 50]"

	switch $var(format) {
	    cds {CATCDS $varname}
	    sdss {CATSDSS $varname}
	}
    } else {
	Info "Please specify width, height, and either name or (ra,dec)"

	$var(mb).file entryconfig $menu(file,retrieve) -state normal
	$var(mb).file entryconfig $menu(file,cancel) -state disabled

	$var(apply) configure -state normal
	$var(cancel) configure -state disabled
    }
}

proc CATLoad {varname} {
    upvar #0 $varname var
    global $varname

    # clear previous db
    global $var(catdb)
    catch {unset $var(catdb)}

    global http

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoad $varname $var(url)?$var(query)"
    }

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url)?$var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(reader) $var(catdb)] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    CATLoadFinish $varname $var(token)
	} else {
	    ARError $varname "Unable to locate URL $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url)?$var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(reader) $var(catdb)] \
					 -command \
					 [list CATLoadFinish $varname]\
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    ARError $varname "Unable to locate URL $var(url)"
	}
    }
}

proc CATLoadFinish {varname token} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoadFinish $varname"
    }

    if {!($var(active))} {
	ARCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	200 -
	203 -
	404 -
	503 {
	    ARDone $varname

	    CATSortMenu $varname
	    CATConfigRADEC $varname
	    CATRADECMenu $varname
	    CATTable $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    if {$debug(tcl,cat)} {
			puts "CATLoadFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    CATLoad $varname
		}
	    }
	}

	default {ARError $varname "Error code $code was returned"}
    }
}

proc CATTable {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global cat
    global current

    global debug
    if {$debug(tcl,cat)} {
	puts "CATTable $varname"
    }

    if {![CATValidDB $var(catdb)]} {
	Error "No catalog data loaded"
	return
    }

    # delete any previous tbldb
    set db ${varname}tbldb
    global $db
    catch {unset $db}

    # clear the selection
    $var(tbl) selection clear all

    # clear regions
    if {$current(frame) != {}} {
	$current(frame) marker catalog $varname delete
    }

    if {$var(filter) == {} && $var(sort) == {}} {
	set var(tbldb) $var(catdb)
    } else {
	set var(tbldb) ${varname}tbldb
	global $var(tbldb)
	if {![CATFltSort $varname]} {
	    Error "Unable to evaluate filter $var(filter)"
	    catch {unset $var(tbldb)}
	    set var(tbldb) $var(catdb)
	}
    }

    global $var(tbldb)
    $var(tbl) configure -variable $var(tbldb)
    $var(found) configure -textvariable ${var(tbldb)}(Nrows)

#    starbase_writefp $var(catdb) stdout
#    starbase_writefp $var(tbldb) stdout

    if {[starbase_nrows $var(tbldb)] == 0} {
	Info "No Items Found"
	return
    }

    set nr [starbase_nrows $var(tbldb)]
    set nc [starbase_ncols $var(tbldb)]

    if { $nc > $cat(mincols)} {
	$var(tbl) configure -cols $nc
    } else {
	$var(tbl) configure -cols $cat(mincols)
    }

    if {$nr > $cat(minrows)} {
	$var(tbl) configure -rows $nr
    } else {
	$var(tbl) configure -rows $cat(minrows)
    }

    CATPlot $varname

    if {$nr >= $var(max) && !$var(allrows)} {
	Info "$nr rows of data have been downloaded. More may be available. You may wish to adjust the maximum allowed."
    }
}

proc CATPlot {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATPlot $varname"
    }

    global current
    if {$current(frame) == {}} {
	return
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname {Plotting Regions...}

    if {[$current(frame) has wcs equatorial wcs]} {
	# delete any previous 
	$current(frame) marker catalog $varname delete

	# load
	global reg
	set reg [CATReg $varname 1]
	if {$reg != {}} {
	    $current(frame) marker catalog command ds9 var reg
	}
    }
    ARStatus $varname {Done}
}

proc CATPlotRegions {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATRegions $varname"
    }

    global current
    if {$current(frame) == {}} {
	return
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname {Generating Regions...}

    if {[$current(frame) has wcs equatorial wcs]} {
	# load
	global reg
	set reg [CATReg $varname 0]
	if {$reg != {}} {
	    $current(frame) marker command ds9 var reg
	}
    }

    ARStatus $varname {Done}
}

proc CATSet {varname format catalog title} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATSet $varname :$format:$catalog:$title:"
    }

    set var(format) $format
    set var(catalog) $catalog
    set var(title) $title
    set var(filter) {}
    set var(ra) {}
    set var(dec) {}
    set var(sort) {}
    set var(sort,dir) "-increasing"
}

proc CATValidDB {varname} {
    upvar #0 $varname var
    global $varname

    if {[info exists var(Nrows)] && 
	[info exists var(Ncols)] &&
	[info exists var(Header)]} {
	return 1
    } else {
	return 0
    }
}

proc CATCatalogMenu {varname} {
    upvar #0 $varname var
    global $varname
    global cat
    global ds9

    foreach mm $cat(def) {
	set ll [lindex $mm 0]
	set ww [lindex $mm 1]
	set ss [lindex $mm 2]
	set cc [lindex $mm 3]

	if {$varname != {}} {
	    if {$ll != "-"} {
		$var(mb).file.cat add command \
		    -label $ll \
		    -command [list CATCatalogCmd $varname $ss $cc $ll]
	    } else {
		$var(mb).file.cat add separator
	    }
	} else {
	    if {$ll != "-"} {
		$ds9(mb).analysis.cat add command \
		    -label $ll -command [list CATDialog $ww $ss $cc $ll apply]
	    } else {
		$ds9(mb).analysis.cat add separator
	    }
	}
    }

    if {$varname != {}} {
	$var(mb).file.cat add separator
	$var(mb).file.cat add command -label "Other Catalogs..." \
	    -command "CATCDSSearch $varname"
    } else {
	$ds9(mb).analysis.cat add separator
	$ds9(mb).analysis.cat add command -label "Other Catalogs..." \
	    -command "CATOther search"
    }
}

proc CATCatalogCmd {varname format catalog title} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCatalogCmd $varname :$format:$catalog:$title:"
    }

    CATOff $varname
    CATSet $varname $format $catalog $title

    CATServerMenu $varname
    CATApply $varname 0
}

proc CATServerMenu {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATServerMenu $varname"
    }

    switch $var(format) {
	sdss {
	    $var(mb) entryconfig "Catalog Server" -state disabled
	}
	default {
	    $var(mb) entryconfig "Catalog Server" -state normal
	}
    }
}

proc CATSortMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global menu

    set m $var(top).param.f.sort.m
    catch {destroy $m}

    menu $m -tearoff 0 -selectcolor $menu(selectcolor)
    $m add command -label {} -command "CATSortCmd $varname {}"
    if [CATValidDB $var(catdb)] {
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATSortCmd $varname \{$col\}"
	}
    }
}

proc CATSortCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(sort) $val
    CATTable $varname
}

proc CATRADECMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global menu

    set m $var(top).param.f.ra.m
    catch {destroy $m}

    menu $m -tearoff 0 -selectcolor $menu(selectcolor)
    if [CATValidDB $var(catdb)] {
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATRACmd $varname \{$col\}"
	}
    }

    set m $var(top).param.f.dec.m
    catch {destroy $m}

    menu $m -tearoff 0 -selectcolor $menu(selectcolor)
    if [CATValidDB $var(catdb)] {
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATDECCmd $varname \{$col\}"
	}
    }
}

proc CATRACmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(ra) $val
    CATPlot $varname
}

proc CATDECCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(dec) $val
    CATPlot $varname
}

proc CATConfigRADEC {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    set var(ra) {}
    set var(dec) {}

    if {![CATValidDB $var(catdb)]} {
	return
    }

    if {[starbase_ncols $var(catdb)] < 2} {
	return
    }

    set var(ra) "_RAJ2000"
    set var(dec) "_DEJ2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "_RAB2000"
    set var(dec) "_DEB2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "RAJ2000"
    set var(dec) "DEJ2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "RA"
    set var(dec) "DEC"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "ra"
    set var(dec) "dec"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    # default
    set var(ra) [starbase_colname $var(catdb) 1]
    set var(dec) [starbase_colname $var(catdb) 2]

    return
}

# Tcl Commands

proc CATButton {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    set marker(motion) none

    # else, see if we are on a marker
    set id [$which get marker catalog id $x $y]
    if {$id != 0} {
	$which marker catalog $id highlite only
	$which marker catalog $id move back
	return
   }

    # see if any markers are highlited
    if {[$which get marker catalog highlite number]>0} {
	$which marker catalog unhighlite all
	return
    }
}

proc CATShift {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }


    if {[$which marker catalog highlite toggle $x $y]} {
	set marker(motion) none
	return
    }

    # else, start a region highlite
    $which region highlite begin $x $y
    set marker(motion) shiftregion
}

proc CATMotion {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    switch -- $marker(motion) {
	none {}
	region -
	shiftregion {$which region highlite motion $x $y}
    }
}

proc CATRelease {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    switch -- $marker(motion) {
	none {}
	region {$which region highlite end}
	shiftregion {$which region highlite shift end}
    }

    unset marker(motion)
}

# Callbacks

proc CATHighliteCB {tag id} {
    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]

    upvar #0 $varname var
    global $varname

    if [info exists ${varname}(top)] {
	if {$var(blink) == 0} {
	    if [info exists ${varname}(tbl)] {
		$var(tbl) selection set $row,1
		$var(tbl) see $row,1
	    }
	}
    }
}

proc CATUnhighliteCB {tag id} {
    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]

    upvar #0 $varname var
    global $varname

    if [info exists ${varname}(top)] {
	if {$var(blink) == 0} {
	    if [info exists ${varname}(tbl)] {
		$var(tbl) selection clear $row,1
	    }
	}
    }
}

proc CATSelectCB {varname} {
    upvar #0 $varname var
    global $varname

    global current
    if {$current(frame) == {}} {
	return
    }

    $current(frame) marker catalog $varname unhighlite

    # init timer vars
    set var(blink,count) 0
    set var(blink,marker) {}

    set row 0
    foreach ss [$var(tbl) curselection] {
	set rr [lindex [split $ss ,] 0]
	if {$rr != $row} {
	    lappend ${varname}(blink,marker) "\{${varname}.${rr}\}"
	    set row $rr
	}
    }

    # start timer, if needed
    if {!$var(blink)} {
	set var(blink) 1
	CATSelectTimer $varname
    }
}

proc CATSelectTimer {varname} {
    upvar #0 $varname var
    global $varname

    global current

    if {$current(frame) == {}} {
	set var(blink) 0
	set var(blink,count) 0
	set var(blink,marker) {}
	return
    }

    switch -- $var(blink) {
	0 {
	    set var(blink) 0
	    set var(blink,count) 0
	    set var(blink,marker) {}
	}
	1 {
	    foreach mm $var(blink,marker) {
		$current(frame) marker catalog $mm highlite
	    }

	    incr ${varname}(blink,count)
	    if {$var(blink,count) < 5} {
		set var(blink) 2
	    } else {
		set var(blink) 0
	    }

	    after 250 [list CATSelectTimer $varname]
	}
	2 {
	    foreach mm $var(blink,marker) {
		$current(frame) marker catalog $mm unhighlite
	    }
	    set var(blink) 1

	    after 250 [list CATSelectTimer $varname]
	}
    }
}

# Other interface

proc CATOther {action} {
    global cat

    incr cat(id)
    set varname "catoth$cat(id)"
    CATDialog $varname {} {} {} $action
}

proc CATClearFrame {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) marker catalog delete all
    }
}

proc CATEditDialog {varname which db} {
    upvar #0 $varname var
    global $varname
    global ed
    global menu

    set w ".${varname}edit"
    set mb ".${varname}editmb"

    set ed(ok) 0
    set ed(text) $w.param.txt

    DialogCreate $w Edit -borderwidth 2

    $w configure -menu $mb
    menu $mb -tearoff 0

    # file
    $mb add cascade -label $menu(file) -menu $mb.file
    menu $mb.file -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.file add command -label $menu(file,apply) -command {set ed(ok) 1}
    $mb.file add separator
    $mb.file add command -label $menu(file,load) -command "CATEditLoad"
    $mb.file add command -label $menu(file,save) -command "CATEditSave"
    $mb.file add separator
    $mb.file add command -label $menu(file,cancel) -command {set ed(ok) 0}

    # edit
    $mb add cascade -label $menu(edit) -menu $mb.edit
    menu $mb.edit -tearoff 0
    $mb.edit add command -label $menu(edit,undo) \
	-command "$ed(text) edit undo"
    $mb.edit add command -label $menu(edit,redo) \
	-command "$ed(text) edit redo"
    $mb.edit add separator
    $mb.edit add command -label $menu(edit,cut) \
	-command "tk_textCut $ed(text)"
    $mb.edit add command -label $menu(edit,copy) \
	-command "tk_textCopy $ed(text)"
    $mb.edit add command -label $menu(edit,paste) \
	-command "tk_textPaste $ed(text)"

    global $db
    # column
    $mb add cascade -label Column -menu $mb.col
    catch {destroy $mb.col}
    menu $mb.col -tearoff 0 -selectcolor $menu(selectcolor)
    if [CATValidDB $db] {
	foreach col [starbase_columns $db] {
	    $mb.col add command -label "$col" \
		-command "$ed(text) insert insert \{\$$col\}"
	}
    }

    # operator
    $mb add cascade -label Operator -menu $mb.op
    menu $mb.op -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add command -label {!} \
	-command "$ed(text) insert insert {!}"
    $mb.op add command -label {(} \
				   -command "$ed(text) insert insert {(}"
	$mb.op add command -label {)} \
	-command "$ed(text) insert insert {)}"
    $mb.op add separator
    $mb.op add command -label {*} \
	-command "$ed(text) insert insert {*}"
    $mb.op add command -label {/} \
	-command "$ed(text) insert insert {/}"
    $mb.op add command -label {%} \
	-command "$ed(text) insert insert {%}"
    $mb.op add command -label {+} \
	-command "$ed(text) insert insert {+}"
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add separator
    $mb.op add command -label {<} \
	-command "$ed(text) insert insert {<}"
    $mb.op add command -label {>} \
	-command "$ed(text) insert insert {>}"
    $mb.op add command -label {<=} \
	-command "$ed(text) insert insert {<=}"
    $mb.op add command -label {>=} \
	-command "$ed(text) insert insert {>=}"
    $mb.op add command -label {==} \
	-command "$ed(text) insert insert {==}"
    $mb.op add command -label {!=} \
	-command "$ed(text) insert insert {!=}"
    $mb.op add separator
    $mb.op add command -label {&&} \
	-command "$ed(text) insert insert {&&}"
    $mb.op add command -label {||} \
	-command "$ed(text) insert insert {||}"

    # operator
    $mb add cascade -label "Math Function" -menu $mb.math
    menu $mb.math -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.math add command -label {acos} \
	-command "$ed(text) insert insert {acos()}"
    $mb.math add command -label {asin} \
	-command "$ed(text) insert insert {asin()}"
    $mb.math add command -label {atan} \
	-command "$ed(text) insert insert {atan()}"
    $mb.math add command -label {atan2} \
	-command "$ed(text) insert insert {atan2(,)}"
    $mb.math add command -label {ceil} \
	-command "$ed(text) insert insert {ceil()}"
    $mb.math add command -label {cos} \
	-command "$ed(text) insert insert {cos()}"
    $mb.math add command -label {cosh} \
	-command "$ed(text) insert insert {cosh()}"
    $mb.math add command -label {exp} \
	-command "$ed(text) insert insert {exp()}"
    $mb.math add command -label {floor} \
	-command "$ed(text) insert insert {floor()}"
    $mb.math add command -label {fmod} \
	-command "$ed(text) insert insert {fmod(,)}"
    $mb.math add command -label {hypot} \
	-command "$ed(text) insert insert {hypot(,)}"
    $mb.math add command -label {log} \
	-command "$ed(text) insert insert {log()}"
    $mb.math add command -label {log10} \
	-command "$ed(text) insert insert {log10()}"
    $mb.math add command -label {pow} \
	-command "$ed(text) insert insert {pow(,)}"
    $mb.math add command -label {sin} \
	-command "$ed(text) insert insert {sin()}"
    $mb.math add command -label {sinh} \
	-command "$ed(text) insert insert {sinh()}"
    $mb.math add command -label {sqrt} \
	-command "$ed(text) insert insert {sqrt()}"
    $mb.math add command -label {tan} \
	-command "$ed(text) insert insert {tan()}"
    $mb.math add command -label {tanh} \
	-command "$ed(text) insert insert {tanh()}"
    $mb.math add command -label {abs} \
	-command "$ed(text) insert insert {abs()}"
    $mb.math add command -label {double} \
	-command "$ed(text) insert insert {double()}"
    $mb.math add command -label {int} \
	-command "$ed(text) insert insert {int()}"
    $mb.math add command -label {round} \
	-command "$ed(text) insert insert {round()}"

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    text $w.param.txt \
	-height 10 \
	-width 60 \
	-font {courier 12} \
	-yscrollcommand "$w.param.yscroll set" \
	-xscrollcommand "$w.param.xscroll set" \
	-undo true \
	-wrap none 

    scrollbar $w.param.yscroll -command [list $ed(text) yview] \
	-orient vertical
    scrollbar $w.param.xscroll -command [list $ed(text) xview] \
	-orient horizontal

    grid $ed(text) $w.param.yscroll -sticky news
    grid $w.param.xscroll -stick news
    grid rowconfigure $w.param 0 -weight 1
    grid columnconfigure $w.param 0 -weight 1

    button $w.buttons.ok -text $menu(file,ok) \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text $menu(file,cancel) \
	-command {set ed(ok) 0}
    pack $w.buttons.ok $w.buttons.cancel -side left -padx 10 -expand true

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    $ed(text) insert end $var($which)
    $ed(text) see end

    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok

    if {$ed(ok)} {
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	set var($which) [string trim $flt]
    }

    DialogDismiss $w
    catch {destroy $mb}
    unset ed
}

proc CATEditSave {} {
    global ed

    set fn [SaveFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn w} fp] {
	    Error "Unable to open $fn: $fp"
	    return
	}
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	puts $fp [string trim $flt]
	catch {close $fp}
    }
}

proc CATEditLoad {} {
    global ed

    set fn [OpenFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn r} fp] {
	    Error "Unable to open $fn: $fp"
	    return
	}
	$ed(text) delete 1.0 end
	$ed(text) insert end [read -nonewline $fp]
	$ed(text) see end
	catch {close $fp}
    }
}

# Process Cmds

proc ProcessCatalogCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global cat

    # we need to be realized
    RealizeDS9

    set cvarname {}

    # determine catalog name
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	cds {
	    # see if its from our list of cats
	    incr i
	    set item [string tolower [lindex $var $i]]
	    foreach mm $cat(def) {
		set ll [lindex $mm 0]
		set ww [lindex $mm 1]
		set ss [lindex $mm 2]
		set cc [lindex $mm 3]
		
		if {"cat${item}" == $ww} {
		    CATDialog $ww $ss $cc $ll sync
		    return
		}
	    }

	    # not a default, assume other name
	    incr cat(id)
	    set catname "catoth$cat(id)"
	    CATDialog $catname cds $item $item sync
	    return
	}

	sdss {
	    set mm [lindex $cat(def) 0]
	    set ll [lindex $mm 0]
	    set ww [lindex $mm 1]
	    set ss [lindex $mm 2]
	    set cc [lindex $mm 3]

	    CATDialog $ww $ss $cc $ll sync
	    return
	}

	file -
	load {
	    incr i
	    set reader starbase_read
	    if {[lindex $var $i] == "csv"} {
		set reader CATCSVRead
		incr i
	    }

	    set fn [lindex $var $i]
	    if {$fn != {}} {
		incr cat(id)
		set catname "catoth$cat(id)"
		CATDialog $catname {} {} {} none
		CATLoadFn $catname $fn $reader
		FileLastFull catfbox $fn
	    }
	    return
	}

	name -
	coordinate -
	size -
	save -
	header -
	filter -
	clear -
	retreive -
	cancel -
	print -
	close -
	server -
	symbol -
	sort -
	maxrows -
	allcols -
	ra -
	dec {set cvarname [lindex $cat(cats) end]}

	default {
	    # see if its from our list of cats
	    set item [string tolower [lindex $var $i]]
	    foreach mm $cat(def) {
		set ll [lindex $mm 0]
		set ww [lindex $mm 1]
		set ss [lindex $mm 2]
		set cc [lindex $mm 3]
		
		if {"cat${item}" == $ww} {
		    CATDialog $ww $ss $cc $ll sync
		    return
		}
	    }

	    # else its an existing cat name
	    set cvarname [lindex $var $i]
	    incr i
	}
    }

    # we should have a catalog now
    global $cvarname
    upvar #0 $cvarname cvar

    if {![info exists cvar(top)]} {
	Error "Unable to find catalog window $cvarname"
	return
    }
    if {![winfo exists $cvar(top)]} {
	Error "Unable to find catalog window $cvarname"
	return
    }

    # now, process it
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	name {
	    incr i
	    set cvar(name) [lindex $var $i]
	}
	coordinate {
	    incr i
	    set cvar(x) [lindex $var $i]
	    incr i
	    set cvar(y) [lindex $var $i]
	    incr i
	    set cvar(sky) [lindex $var $i]
	}
	size {
	    incr i
	    set cvar(width) [lindex $var $i]
	    incr i
	    set cvar(height) [lindex $var $i]
	    incr i
	    set cvar(rformat) [lindex $var $i]
	}
	save {
	    global $cvar(tbldb)
	    incr i
	    starbase_write $cvar(tbldb) [lindex $var $i]
	}
	header {}
	filter {
	    incr i
	    set item [lindex $var $i]
	    switch -- $item {
		load {
		    incr i
		    set fn [lindex $var $i]
		    if [catch {open $fn r} fp] {
			Error "Unable to open $fn: $fp"
			return
		    }
		    set flt [read -nonewline $fp]
		    catch {regsub {\n} $flt " " $flt}
		    set cvar(filter) [string trim $flt]
		    catch {close $fp}
		}
		default {
		    set cvar(filter) $item
		}
	    }
	    CATTable $cvarname
	}
	clear {CATOff $cvarname}
	retreive {CATApply $cvarname 0}
	cancel {ARCancel $cvarname}
	print {CATPrint $cvarname}
	close {CATDestroy $cvarname}
	server {
	    incr i
	    set cvar(server) [lindex $var $i]
	}
	symbol {
	    global $cvar(symdb)
	    incr i
	    switch -- [lindex $var $i] {
		shape {
		    incr i
		    starbase_set $cvar(symdb) 1 \
			[starbase_colnum $cvar(symdb) shape] [lindex $var $i]
		}
		color {
		    incr i
		    starbase_set $cvar(symdb) 1 \
			[starbase_colnum $cvar(symdb) color] [lindex $var $i]
		}
		load {
		    incr i
		    starbase_read $cvar(symdb) [lindex $var $i]
		}
	    }
	    CATPlot $cvarname
	}
	sort {
	    incr i
	    set cvar(sort) [lindex $var $i]
	    incr i
	    switch -- [lindex $var $i] {
		incr {
		    set cvar(sort,dir) "-increasing"
		}
		decr {
		    set cvar(sort,dir) "-decreasing"
		}
	    }
	    CATTable $cvarname
	}
	maxrows {
	    incr i
	    set cvar(max) [lindex $var $i]
	}
	allrows {
	    set cvar(allrows) 1
	}
	ra {
	    incr i
	    set cvar(ra) [lindex $var $i]
	    CATPlot $cvarname
	}
	dec {
	    incr i
	    set cvar(dec) [lindex $var $i]
	    CATPlot $cvarname
	}
    }
}
