#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc DSSDialog {} {
    global dss
    global menu

    if [winfo exists $dss(top)] {
	raise $dss(top)
	return
    }

    set varname dssi
    upvar #0 $varname var
    global $varname

    set var(top) $dss(top)
    set var(mb) $dss(mb)
    set var(server) $dss(server)
    set var(system) $dss(system)
    set var(sky) $dss(sky)
    set var(skyformat) $dss(skyformat)
    set var(rformat) $dss(rformat)
    set var(width) $dss(width)
    set var(height) $dss(height)
    set var(survey) $dss(survey)
    set var(mode) $dss(mode)
    set var(save) $dss(save)
    set var(valid) $dss(valid)

    set w $var(top)
    IMGSVRInit $varname "DSS Server" DSSExec DSSAck DSSUpdate

    # custom dialog
    label $w.param.wtitle -text "Width"
    entry $w.param.w -textvariable ${varname}(width) -width 14
    label $w.param.htitle -text "Height"
    entry $w.param.h -textvariable ${varname}(height) -width 14
    label $w.param.format -textvariable ${varname}(rformat) \
	-width 10 -relief groove

    grid $w.param.wtitle $w.param.w $w.param.htitle $w.param.h \
	$w.param.format -padx 4 -pady 1 -sticky w

    # menu
    IMGSVRFileMenu $varname
    AREditMenu $varname
    $var(mb) add cascade -label "DSS Server" -menu $var(mb).server
    NSVRServerMenu $varname
    $var(mb) add cascade -label Survey -menu $var(mb).survey
    IMGSVRPrefsMenu $varname

    menu $var(mb).server -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).server add radiobutton -label "DSS@SAO" \
	-variable ${varname}(server) -value sao \
	-command "DSSServer $varname"
    $var(mb).server add radiobutton -label "DSS@STSCI" \
	-variable ${varname}(server) -value stsci \
	-command "DSSServer $varname"
    $var(mb).server add radiobutton -label "DSS@ESO" \
	-variable ${varname}(server) -value eso \
	-command "DSSServer $varname"

    menu $var(mb).survey -tearoff 0 -selectcolor $menu(selectcolor)

    IMGSVRUpdate $varname 1

    DSSServer $varname
}

proc DSSExec {varname} {
    upvar #0 $varname var
    global $varname

    switch -- $var(server) {
	sao {DSSSAO $varname}
	stsci {DSSSTSCI $varname}
	eso {DSSESO $varname}
    }
}

proc DSSSAO {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set compress no
	set var(fn) [SaveFileDialog dssfbox]
	if {$var(fn) == {}} {
	    return
	}

    } else {
	set compress gzip
	set var(fn) [tmpnam dss ".fits.gz"]
    }

    if {$var(width)>60} {
	set var(width) 60
    }
    if {$var(height)>60} {
	set var(height) 60
    }
    set var(query) [http::formatQuery r $var(x) d $var(y) e J2000 \
			w $var(width) h $var(height) \
			c $compress]
    set var(url) "http://archive.harvard.edu/archive/dss"
    IMGSVRLoad $varname
}

proc DSSESO {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set mime "application/x-fits"
	set var(fn) [SaveFileDialog dssfbox]
	if {$var(fn) == {}} {
	    return
	}
    } else {
	set mime "display/gz-fits"
	set var(fn) [tmpnam dss ".fits.gz"]
    }

    if {$var(width)>40} {
	set var(width) 40
    }
    if {$var(height)>40} {
	set var(height) 40
    }
    set query [http::formatQuery ra $var(x) dec $var(y) equinox J2000\
			x $var(width) y $var(height) \
			mime-type "$mime" \
			Sky-Survey $var(survey)]
    set var(query) {}
    # Load image
    # we can't use -query because eso needs a GET not a POST
    set var(url) "http://archive.eso.org/dss/dss?$query"
    IMGSVRLoad $varname
}

proc DSSSTSCI {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set compress none
	set var(fn) [SaveFileDialog dssfbox]
	if {$var(fn) == {}} {
	    return
	}

    } else {
	set compress gz
	set var(fn) [tmpnam dss ".fits.gz"]
    }

    if {$var(width)>60} {
	set var(width) 60
    }
    if {$var(height)>60} {
	set var(height) 60
    }
    set var(query) [http::formatQuery r $var(x) d $var(y) e J2000\
			w $var(width) h $var(height) \
			f fits c $compress \
			v $var(survey)]
    set var(url) "http://stdatu.stsci.edu/cgi-bin/dss_search"
    IMGSVRLoad $varname
}

proc DSSAck {varname} {
    upvar #0 $varname var
    global $varname

set msg {Acknowledgements for the DSS 

The Digitized Sky Surveys were produced at the Space Telescope Science
Institute under U.S.  Government grant NAG W-2166. The images of these
surveys are based on photographic data obtained using the Oschin
Schmidt Telescope on Palomar Mountain and the UK Schmidt Telescope.
The plates were processed into the present compressed digital form
with the permission of these institutions.

The National Geographic Society - Palomar Observatory Sky Atlas
(POSS-I) was made by the California Institute of Technology with
grants from the National Geographic Society.

The Second Palomar Observatory Sky Survey (POSS-II) was made by the
California Institute of Technology with funds from the National
Science Foundation, the National Geographic Society, the Sloan
Foundation, the Samuel Oschin Foundation, and the Eastman Kodak
Corporation.

The Oschin Schmidt Telescope is operated by the California Institute
of Technology and Palomar Observatory.

The UK Schmidt Telescope was operated by the Royal Observatory
Edinburgh, with funding from the UK Science and Engineering Research
Council (later the UK Particle Physics and Astronomy Research
Council), until 1988 June, and thereafter by the Anglo-Australian
Observatory. The blue plates of the southern Sky Atlas and its
Equatorial Extension (together known as the SERC-J), as well as the
Equatorial Red (ER), and the Second Epoch [red] Survey (SES) were all
taken with the UK Schmidt.
}

    SimpleTextDialog dssack Acknowledgement 80 40 insert top $msg
}

proc DSSUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global current

    set size [$current(frame) get fits size \
		  $var(system) $var(sky) $var(rformat)]
    set var(width) [lindex $size 0]
    set var(height) [lindex $size 1]
}

proc DSSServer {varname} {
    upvar #0 $varname var
    global $varname

    $var(mb).survey delete 0 end

    switch -- $var(server) {
	sao {
	    $var(mb) entryconfig "Survey" -state disabled
	    set var(survey) {}
	}
	stsci {
	    set var(survey) {all}

	    $var(mb) entryconfig "Survey" -state normal
	    $var(mb).survey add radiobutton -label "POSS2/UKSTU Red" \
		-variable ${varname}(survey) -value poss2ukstu_red
	    $var(mb).survey add radiobutton -label "POSS2/UKSTU Infrared" \
		-variable ${varname}(survey) -value poss2ukstu_ir
	    $var(mb).survey add radiobutton -label "POSS2/UKSTU Blue" \
		-variable ${varname}(survey) -value poss2ukstu_blue
	    $var(mb).survey add radiobutton \
		-label "POSS1 (First Generation) Blue" \
		-variable ${varname}(survey) -value poss1_blue
	    $var(mb).survey add radiobutton \
		-label "POSS1 (First Generation) Red" \
		-variable ${varname}(survey) -value poss1_red
	    $var(mb).survey add radiobutton \
		-label "Best of a combined list of all plates" \
		-variable ${varname}(survey) -value all
	    $var(mb).survey add radiobutton \
		-label "Quick-V Survey" \
		-variable ${varname}(survey) -value quickv
	    $var(mb).survey add radiobutton \
		-label "HST Phase 2 Target Positioning (GSC 2)" \
		-variable ${varname}(survey) -value phase2_gsc2
	    $var(mb).survey add radiobutton \
		-label "HST Phase 2 Target Positioning (GSC 1)" \
		-variable ${varname}(survey) -value phase2_gsc1
	}
	eso {
	    set var(survey) {dss}

	    $var(mb) entryconfig "Survey" -state normal
	    $var(mb).survey add radiobutton -label "DSS1" \
		-variable ${varname}(survey) -value dss
	    $var(mb).survey add radiobutton -label "DSS2-red" \
		-variable ${varname}(survey) -value dss2red
	    $var(mb).survey add radiobutton -label "DSS2-blue" \
		-variable ${varname}(survey) -value dss2blue
	}
    }
}

proc UpdateDSSDialog {} {
    global dss

    if [winfo exists $dss(top)] {
	IMGSVRUpdate dssi 0
    }
}

# Process Cmds

proc ProcessDSSCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global current

    set vvarname dssi
    upvar #0 $vvarname vvar
    global $vvarname

    # do we have a rgb frame
    if {[$current(frame) get type] == "rgb"} {
	set base 0
    } else {
	set base 1
    }

    DSSDialog

    set vvar(mode) current

    switch -- [string tolower [lindex $var $i]] {
	server {
	    incr i
	    set vvar(server) [lindex $var $i]
	}
	survey {
	    incr i
	    set vvar(survey) [lindex $var $i]
	    switch -- $vvar(survey) {
		dss {}
		dss2red -
		dss2blue {set vvar(server) eso}
		default {set vvar(server) stsci}
	    }
	}
	size {
	    incr i
	    set vvar(width) [lindex $var $i]
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	width {
	    incr i
	    set vvar(width) [lindex $var $i]
	}
	height {
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	coord {
	    if {$flag} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(x) [lindex $var $i]
	    incr i
	    set vvar(y) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	x {
	    incr i
	    set vvar(x) [lindex $var $i]
	}
	y {
	    incr i
	    set vvar(y) [lindex $var $i]
	}
	name {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	default {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
    }
}

