#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc FIRSTDialog {} {
    global first
    global menu

    if [winfo exists $first(top)] {
	raise $first(top)
	return
    }

    set varname firsti
    upvar #0 $varname var
    global $varname

    set var(top) $first(top)
    set var(mb) $first(mb)
    set var(system) $first(system)
    set var(sky) $first(sky)
    set var(skyformat) $first(skyformat)
    set var(rformat) $first(rformat)
    set var(radius) $first(radius)
    set var(mode) $first(mode)
    set var(save) $first(save)
    set var(valid) $first(valid)

    set w $var(top)
    IMGSVRInit $varname "FIRST Server" FIRSTExec FIRSTAck FIRSTUpdate

    # custom dialog
    label $w.param.wtitle -text "Radius"
    entry $w.param.w -textvariable ${varname}(radius) -width 14
    label $w.param.format -textvariable ${varname}(rformat) \
	-width 10 -relief groove

    grid $w.param.wtitle $w.param.w x x $w.param.format \
	-padx 4 -pady 1 -sticky w

    # menu
    IMGSVRFileMenu $varname
    AREditMenu $varname
    NSVRServerMenu $varname
    IMGSVRPrefsMenu $varname

    IMGSVRUpdate $varname 1
}

proc FIRSTExec {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set var(fn) [SaveFileDialog firstfbox]
	if {$var(fn) == {}} {
	    return
	}

    } else {
	set var(fn) [tmpnam first ".fits"]
    }

    if {$var(radius)>3600} {
	set var(radius) 3600
    }
    set foo "$var(x) $var(y)"

    set var(query) [http::formatQuery \.submit \ Extract\ the\ Cutout RA $var(x)\ $var(y) Equinox J2000 ImageSize $var(radius) MaxInt 10 \.cgifields ImageType ImageType FITS\ Image]
    set var(url) "http://third.ucllnl.org/cgi-bin/firstcutout"

    IMGSVRLoad $varname
}

proc FIRSTAck {varname} {
    upvar #0 $varname var
    global $varname

set msg {Acknowledgements for the VLA FIRST 

This major undertaking has received the generous technical and
scientific support of many individuals. The NRAO staff has provided
extremely valuable assistance in many aspects of the observations
themselves and in the area of software support; in particular, we are
grateful to Rick Perley, Ken Sowinski, Barry Clark, and Bill Cotton in
this regard. The support of the NRAO Director, Paul van den Bout, and
the yeoman service provided by Frazer Owen as Chair of the Survey
Oversight Committee are also greatly appreciated. We also thank the
members of the Oversight Committee (Ken Chambers, Eric Feigelson,
Jackie Hewitt, Gillian Knapp, and Rogier Windhorst) for their time and
wise counsel in this enterprise.

Acknowledgement is also due our colleagues who are involved in the
ongoing FIRST effort, including Richard McMahon and Isobel Hook. This
work is supported in part under the auspices of the Department of
Energy by Lawrence Livermore National Laboratory under contract
No. W-7405-ENG-48 and the Institute for Geophysics and Planetary
Physics, whose director Charles Alcock has been particularly
supportive. We also acknowledge a generous planning grant from the
CalSpace Institute; support from the STScI archive group, STScI
director Bob Williams, and the STScI Director's Discretionary Research
Fund; computing resources from Columbia University; a grant from the
National Science Foundation; a gift of computing equipment from Sun
Microsystems; a NATO travel grant to support our collaboration with
Richard McMahon; and an award from the National Geographic Society
which, in the spirit of their support 40 years ago for the Palomar
Observatory Sky Survey, will be providing funds to continue our
charting of the Universe.
}

    SimpleTextDialog firstack Acknowledgement 80 40 insert top $msg
}

proc FIRSTUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global current

    set size [$current(frame) get fits size \
		  $var(system) $var(sky) $var(rformat)]
    set var(radius) [lindex $size 0]
}

proc UpdateFIRSTDialog {} {
    global first

    if [winfo exists $first(top)] {
	IMGSVRUpdate firsti 0
    }
}

# Process Cmds

proc ProcessFIRSTCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global current

    set vvarname firsti
    upvar #0 $vvarname vvar
    global $vvarname

    # do we have a rgb frame
    if {[$current(frame) get type] == "rgb"} {
	set base 0
    } else {
	set base 1
    }

    FIRSTDialog
    set vvar(mode) current

    switch -- [string tolower [lindex $var $i]] {
	size {
	    incr i
	    set vvar(radius) [lindex $var $i]
	}
	coord {
	    if {$flag} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(x) [lindex $var $i]
	    incr i
	    set vvar(y) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	x {
	    incr i
	    set vvar(x) [lindex $var $i]
	}
	y {
	    incr i
	    set vvar(y) [lindex $var $i]
	}
	name {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	default {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
    }
}
