#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc AnalysisMenu {} {
    global ds9
    global menu

    set menu(analysis,pixel) [msgcat::mc "Pixel Table..."]
    set menu(analysis,contour) [msgcat::mc "Contours"]
    set menu(analysis,contour,params) [msgcat::mc "Contours Parameters..."]
    set menu(analysis,grid) [msgcat::mc "Coordinate Grid"]
    set menu(analysis,grid,params) [msgcat::mc "Coordinate Grid Parameters..."]
    set menu(analysis,smooth) [msgcat::mc "Smooth"]
    set menu(analysis,smooth,params) [msgcat::mc "Smooth Parameters..."]
    set menu(analysis,names) [msgcat::mc "Name Resolution..."]
    set menu(analysis,images) [msgcat::mc "Image Servers"]
    set menu(analysis,archives) [msgcat::mc "Archives"]
    set menu(analysis,catalogs) [msgcat::mc "Catalogs"]
    set menu(analysis,vo) [msgcat::mc "Virtual Observatory..."]
    set menu(analysis,plot) [msgcat::mc "Plot Tool..."]
    set menu(analysis,web) [msgcat::mc "Web Browser..."]
    set menu(analysis,catalog) [msgcat::mc "Catalog Tool..."]
    set menu(analysis,log) [msgcat::mc "Analysis Command Log"]
    set menu(analysis,load) [msgcat::mc "Load Analysis Commands..."]
    set menu(analysis,clear) [msgcat::mc "Clear Analysis Commands"]

    set menu(analysis,catalogs,clear) [msgcat::mc "Clear All"]

    # WARNING: this is a variable length menu. 
    # Be sure to update menu(size,analysis)
    menu $ds9(mb).analysis -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Analysis"
    $ds9(mb).analysis add command -label $menu(analysis,pixel) \
	-command PTDialog
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add checkbutton -label $menu(analysis,contour) \
	-variable contour(view) -command ContourUpdate
    $ds9(mb).analysis add command -label $menu(analysis,contour,params) \
	-command ContourDialog
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add checkbutton -label $menu(analysis,grid) \
	-variable grid(view) -command GridUpdate
    $ds9(mb).analysis add command -label $menu(analysis,grid,params) \
	-command GridDialog
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add checkbutton -label $menu(analysis,smooth) \
	-variable smooth(view) -command SmoothUpdate
    $ds9(mb).analysis add command -label $menu(analysis,smooth,params) \
	-command SmoothDialog
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add command -label $menu(analysis,names) \
	-command NRESDialog
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add cascade -label $menu(analysis,images) \
	-menu $ds9(mb).analysis.image
    $ds9(mb).analysis add cascade -label $menu(analysis,archives) \
	-menu $ds9(mb).analysis.arch
    $ds9(mb).analysis add cascade -label $menu(analysis,catalogs) \
	-menu $ds9(mb).analysis.cat
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add command -label $menu(analysis,vo) \
	-command "VODialog 0"
    $ds9(mb).analysis add command -label $menu(analysis,plot) \
	-command AnalysisPlotTool
    $ds9(mb).analysis add command -label $menu(analysis,web) \
	-command {HV web Web {} 2 1 {}}
    $ds9(mb).analysis add command -label $menu(analysis,catalog) \
	-command {CATOther none}
    $ds9(mb).analysis add separator
    $ds9(mb).analysis add checkbutton -label $menu(analysis,log) \
	-variable analysis(log)
    $ds9(mb).analysis add command -label $menu(analysis,load) \
	-command LoadAnalysisDialog
    $ds9(mb).analysis add command -label $menu(analysis,clear) \
	-command ClearAnalysisDialog

    menu $ds9(mb).analysis.image -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Analysis Image"
    $ds9(mb).analysis.image add command \
	-label "Digital Sky Survey (DSS)" \
	-command DSSDialog
    $ds9(mb).analysis.image add command \
	-label "Two Micron All Sky Survey (2MASS)" \
	-command 2MASSDialog
    $ds9(mb).analysis.image add command \
	-label "VLA FIRST Survey (VLA)" \
	-command FIRSTDialog

    menu $ds9(mb).analysis.cat -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Analysis Catalog"
    CATCatalogMenu {}
    $ds9(mb).analysis.cat add separator
    $ds9(mb).analysis.cat add command -label $menu(analysis,catalogs,clear) \
	-command CATClearFrame

    # Analysis Archive Menu
    # WARNING: this is a variable length menu. 
    # Be sure to update menu(size,arch)
    menu $ds9(mb).analysis.arch -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Archive Servers"
    $ds9(mb).analysis.arch add command -label "NVO DataScope" \
	-command HVArchDataScope
    $ds9(mb).analysis.arch add separator
    $ds9(mb).analysis.arch add cascade -label "Chandra" \
	-menu $ds9(mb).analysis.arch.chandra
    $ds9(mb).analysis.arch add command -label "ROSAT All-Sky (MPE/MPG)" \
	-command HVArchRosat
    $ds9(mb).analysis.arch add command -label "SkyView (NASA/HEASARC)" \
	-command HVArchSkyView
    $ds9(mb).analysis.arch add command -label "W3Browse (NASA/HEASARC)" \
	-command HVArchW3Browse
    $ds9(mb).analysis.arch add separator
    $ds9(mb).analysis.arch add command -label "NVSS (NRAO)" \
	-command HVArchNVSS
    $ds9(mb).analysis.arch add command -label "4MASS (NRAO)" \
	-command HVArch4MASS
    $ds9(mb).analysis.arch add command -label "SIRTF FLS/VLA (NRAO)" \
	-command HVArchSIRTF
    $ds9(mb).analysis.arch add command -label "VLA First (NRAO)" \
	-command HVArchFirst
    $ds9(mb).analysis.arch add separator
    $ds9(mb).analysis.arch add command -label "NED (NASA/IPAC)" \
	-command HVArchNED
    $ds9(mb).analysis.arch add cascade -label "SIMBAD (CDS)" \
	-menu $ds9(mb).analysis.arch.simbad
    $ds9(mb).analysis.arch add cascade -label "ADS (SAO)" \
	-menu $ds9(mb).analysis.arch.ads
    $ds9(mb).analysis.arch add separator
    $ds9(mb).analysis.arch add command -label "SAO TDC (SAO)" \
	-command HVArchSAOTDC
    $ds9(mb).analysis.arch add separator

    HVArchUserMenu

    menu $ds9(mb).analysis.arch.chandra -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Archive Servers"
    $ds9(mb).analysis.arch.chandra add command \
	-label "Chaser (NASA/SAO)" -command HVArchChandraChaser
    $ds9(mb).analysis.arch.chandra add command \
	-label "Fast Image (NASA/SAO)" -command HVArchChandraPop
    $ds9(mb).analysis.arch.chandra add command \
	-label "Public FTP (SAO)" -command HVArchChandraFTP

    menu $ds9(mb).analysis.arch.simbad -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Archive Servers"
    $ds9(mb).analysis.arch.simbad add command -label "SAO" \
	-command HVArchSIMBADSAO
    $ds9(mb).analysis.arch.simbad add command -label "CDS" \
	-command HVArchSIMBADCDS

    menu $ds9(mb).analysis.arch.ads -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Archive Servers"
    $ds9(mb).analysis.arch.ads add command -label "SAO" \
	-command HVArchADSSAO
    $ds9(mb).analysis.arch.ads add command -label "CDS" \
	-command HVArchADSCDS

    AnalysisPrefsMenu
}

proc AnalysisPrefsMenu {} {
    global ds9
    global menu
    global prefs

    set menu(prefs,analysis,names) [msgcat::mc "Name Server"]
    set menu(prefs,analysis,dss) [msgcat::mc "DSS Server"]
    set menu(prefs,analysis,archives) [msgcat::mc "Archive Server"]
    set menu(prefs,analysis,vo) [msgcat::mc "Virtual Observatory"]
    set menu(prefs,analysis,file) [msgcat::mc "Preload Analysis File"]
    set menu(prefs,analysis,plot) [msgcat::mc "Analysis Plot"]

    set menu(prefs,analysis,contour,color) [msgcat::mc "Color"]
    set menu(prefs,analysis,contour,width) [msgcat::mc "Width"]
    set menu(prefs,analysis,contour,method) [msgcat::mc "Method"]

    set menu(prefs,analysis,contour,method,block) [msgcat::mc "Block"]
    set menu(prefs,analysis,contour,method,smooth) [msgcat::mc "Smooth"]

    set menu(prefs,analysis,smooth,function) [msgcat::mc "Function"]

    set menu(prefs,analysis,catalog,server) [msgcat::mc "Server"]
    set menu(prefs,analysis,catalog,symbol) [msgcat::mc "Symbol"]

    set menu(prefs,analysis,vo,display) [msgcat::mc "Use Internal Web Browser"]
    set menu(prefs,analysis,vo,http) [msgcat::mc "Connect Use Web Proxy"]
    set menu(prefs,analysis,vo,xpa) [msgcat::mc "Connect Directly"]
    set menu(prefs,analysis,vo,server) [msgcat::mc "Select VO Server..."]

    # Analysis Prefs
    menu $ds9(mb).prefs.gui.analysis -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(analysis,contour) \
	-menu $ds9(mb).prefs.gui.analysis.contour
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(analysis,smooth) \
	-menu $ds9(mb).prefs.gui.analysis.smooth
    $ds9(mb).prefs.gui.analysis add separator
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,names) \
	-menu $ds9(mb).prefs.gui.analysis.namesvr
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,dss) \
	-menu $ds9(mb).prefs.gui.analysis.dsssvr
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(analysis,catalogs) \
	-menu $ds9(mb).prefs.gui.analysis.cat
    $ds9(mb).prefs.gui.analysis add separator
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,archives) \
	-menu $ds9(mb).prefs.gui.analysis.archive
    $ds9(mb).prefs.gui.analysis add separator
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,vo) \
	-menu $ds9(mb).prefs.gui.analysis.vo
    $ds9(mb).prefs.gui.analysis add separator
    $ds9(mb).prefs.gui.analysis add checkbutton -label $menu(analysis,log) \
	-variable prefs(analysis,log) -command SavePrefs
    $ds9(mb).prefs.gui.analysis add separator
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,file) \
	-menu $ds9(mb).prefs.gui.analysis.preload
    $ds9(mb).prefs.gui.analysis add cascade -label $menu(prefs,analysis,plot) \
	-menu $ds9(mb).prefs.gui.analysis.plot

    # Analysis Contour Prefs
    menu $ds9(mb).prefs.gui.analysis.contour -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.contour add cascade \
	-label $menu(prefs,analysis,contour,color) \
	-menu $ds9(mb).prefs.gui.analysis.contour.color
    $ds9(mb).prefs.gui.analysis.contour add cascade \
	-label $menu(prefs,analysis,contour,width) \
	-menu $ds9(mb).prefs.gui.analysis.contour.width
    $ds9(mb).prefs.gui.analysis.contour add cascade \
	-label $menu(prefs,analysis,contour,method) \
	-menu $ds9(mb).prefs.gui.analysis.contour.method

    # Analysis Contour Color Prefs
    menu $ds9(mb).prefs.gui.analysis.contour.color -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,black) \
	-variable prefs(contour,color) -value black -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,white) \
	-variable prefs(contour,color) -value white  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,red) \
	-variable prefs(contour,color) -value red  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,green) \
	-variable prefs(contour,color) -value green  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,blue) \
	-variable prefs(contour,color) -value blue  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,cyan) \
	-variable prefs(contour,color) -value cyan  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,magenta) \
	-variable prefs(contour,color) -value magenta  -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.color add radiobutton \
	-label $menu(color,yellow) \
	-variable prefs(contour,color) -value yellow  -command SavePrefs

    # Analysis Contour Width Prefs
    menu $ds9(mb).prefs.gui.analysis.contour.width -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.contour.width add radiobutton -label "1" \
	-variable prefs(contour,width) -value 1 -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.width add radiobutton -label "2" \
	-variable prefs(contour,width) -value 2 -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.width add radiobutton -label "3" \
	-variable prefs(contour,width) -value 3 -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.width add radiobutton -label "4" \
	-variable prefs(contour,width) -value 4 -command SavePrefs

    # Analysis Contour Method Prefs
    menu $ds9(mb).prefs.gui.analysis.contour.method -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.contour.method add radiobutton \
	-label $menu(prefs,analysis,contour,method,block) \
	-variable prefs(contour,method) -value block -command SavePrefs
    $ds9(mb).prefs.gui.analysis.contour.method add radiobutton \
	-label $menu(prefs,analysis,contour,method,smooth) \
	-variable prefs(contour,method) -value smooth -command SavePrefs

    # Analysis Smooth Prefs
    menu $ds9(mb).prefs.gui.analysis.smooth -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.smooth add cascade \
	-label $menu(prefs,analysis,smooth,function) \
	-menu $ds9(mb).prefs.gui.analysis.smooth.function

    # Analysis Smooth Function Prefs
    menu $ds9(mb).prefs.gui.analysis.smooth.function -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.smooth.function add radiobutton \
	-label "Boxcar" \
	-variable prefs(smooth,function) -value boxcar -command SavePrefs
    $ds9(mb).prefs.gui.analysis.smooth.function add radiobutton \
	-label "Tophat" \
	-variable prefs(smooth,function) -value tophat -command SavePrefs
    $ds9(mb).prefs.gui.analysis.smooth.function add radiobutton \
	-label "Gaussian" \
	-variable prefs(smooth,function) -value gaussian -command SavePrefs

    # Analysis Name Server Prefs
    menu $ds9(mb).prefs.gui.analysis.namesvr -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.namesvr add radiobutton -label "NED@SAO" \
	-variable nres(server) -value ned-sao -command SavePrefs
    $ds9(mb).prefs.gui.analysis.namesvr add radiobutton -label "NED@ESO" \
	-variable nres(server) -value ned-eso -command SavePrefs
    $ds9(mb).prefs.gui.analysis.namesvr add radiobutton -label "SIMBAD@SAO" \
	-variable nres(server) -value simbad-sao -command SavePrefs
    $ds9(mb).prefs.gui.analysis.namesvr add radiobutton -label "SIMBAD@ESO" \
	-variable nres(server) -value simbad-eso -command SavePrefs

    # Analysis DSS Server Prefs
    menu $ds9(mb).prefs.gui.analysis.dsssvr -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.dsssvr add radiobutton -label "DSS@SAO" \
	-variable dss(server) -value sao -command SavePrefs
    $ds9(mb).prefs.gui.analysis.dsssvr add radiobutton -label "DSS@STSCI" \
	-variable dss(server) -value stsci -command SavePrefs
    $ds9(mb).prefs.gui.analysis.dsssvr add radiobutton -label "DSS@ESO" \
	-variable dss(server) -value eso -command SavePrefs

    # Analysis Catalog Prefs
    menu $ds9(mb).prefs.gui.analysis.cat -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.cat add cascade \
	-label $menu(prefs,analysis,catalog,server) \
	-menu $ds9(mb).prefs.gui.analysis.cat.server
    $ds9(mb).prefs.gui.analysis.cat add cascade \
	-label $menu(prefs,analysis,catalog,symbol) \
	-menu $ds9(mb).prefs.gui.analysis.cat.sym

    # Analysis Catalog Server Prefs
    menu $ds9(mb).prefs.gui.analysis.cat.server -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "CDS" \
	-variable cat(server) -value cds -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "SAO" \
	-variable cat(server) -value sao -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "CADC" \
	-variable cat(server) -value cadc -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "ADAC" \
	-variable cat(server) -value adac -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "IUCAA" \
	-variable cat(server) -value iucaa -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton -label "BEJING" \
	-variable cat(server) -value bejing -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton \
	-label "CAMBRIDGE UK" \
	-variable cat(server) -value cambridge -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.server add radiobutton \
	-label "UKIRT HAWAII" \
	-variable cat(server) -value ukirt -command SavePrefs

    # Analysis Catalog Symbol Prefs
    menu $ds9(mb).prefs.gui.analysis.cat.sym -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,circle) \
	-variable cat(sym,shape) -value {circle point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,box) \
	-variable cat(sym,shape) -value {box point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,diamond) \
	-variable cat(sym,shape) -value {diamond point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,cross) \
	-variable cat(sym,shape) -value {cross point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,x) \
	-variable cat(sym,shape) -value {x point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,arrow) \
	-variable cat(sym,shape) -value {arrow point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,point,boxcircle) \
	-variable cat(sym,shape) -value {boxcircle point} -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add separator
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,circle) \
	-variable cat(sym,shape) -value circle -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,ellipse) \
	-variable cat(sym,shape) -value ellipse -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,box) \
	-variable cat(sym,shape) -value box -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(shape,text) \
	-variable cat(sym,shape) -value text -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add separator
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,white) \
	-variable cat(sym,color) -value white -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,black) \
	-variable cat(sym,color) -value black -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,red) \
	-variable cat(sym,color) -value red -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,green) \
	-variable cat(sym,color) -value green -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,blue) \
	-variable cat(sym,color) -value blue -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,cyan) \
	-variable cat(sym,color) -value cyan -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,magenta) \
	-variable cat(sym,color) -value magenta -command SavePrefs
    $ds9(mb).prefs.gui.analysis.cat.sym add radiobutton \
	-label $menu(color,yellow) \
	-variable cat(sym,color) -value yellow -command SavePrefs

    # Analysis Archive Prefs
    menu $ds9(mb).prefs.gui.analysis.archive -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.archive add command -label "1..." \
	-command {PrefsHVArch 1}
    $ds9(mb).prefs.gui.analysis.archive add command -label "2..." \
	-command {PrefsHVArch 2}
    $ds9(mb).prefs.gui.analysis.archive add command -label "3..." \
	-command {PrefsHVArch 3}
    $ds9(mb).prefs.gui.analysis.archive add command -label "4..." \
	-command {PrefsHVArch 4}

    # Analysis VO Prefs
    menu $ds9(mb).prefs.gui.analysis.vo -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.vo add checkbutton \
	-label $menu(prefs,analysis,vo,display) \
	-variable vo(hv) -command SavePrefs
    $ds9(mb).prefs.gui.analysis.vo add separator
    $ds9(mb).prefs.gui.analysis.vo add radiobutton \
	-label $menu(prefs,analysis,vo,http) \
	-variable vo(method) -value mime -command PrefsVOMethod
    $ds9(mb).prefs.gui.analysis.vo add radiobutton \
	-label $menu(prefs,analysis,vo,xpa) \
	-variable vo(method) -value xpa -command PrefsVOMethod
    $ds9(mb).prefs.gui.analysis.vo add separator
    $ds9(mb).prefs.gui.analysis.vo add command \
	-label $menu(prefs,analysis,vo,server) \
	-command PrefsVOServer

    # Analysis Preload Prefs
    menu $ds9(mb).prefs.gui.analysis.preload -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.analysis.preload add command -label "1..." \
	-command {PrefsAnalysisUser ds9(analysis,user)}
    $ds9(mb).prefs.gui.analysis.preload add command -label "2..." \
	-command {PrefsAnalysisUser ds9(analysis,user2)}
    $ds9(mb).prefs.gui.analysis.preload add command -label "3..." \
	-command {PrefsAnalysisUser ds9(analysis,user3)}
    $ds9(mb).prefs.gui.analysis.preload add command -label "4..." \
	-command {PrefsAnalysisUser ds9(analysis,user4)}

    # Analysis Plot Prefs
    PrefsAPMenu
}

proc UpdateAnalysisMenu {} {
    global ds9
    global current
    global analysis
    global menu

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateAnalysisMenu"
    }

    if {$current(frame) != {}} {
	$ds9(mb) entryconfig $menu(analysis) -state normal
	for {set i 0} {$i<$analysis(menu,count)} {incr i} {

	    if {[$current(frame) has fits]} {
		set fn [$current(frame) get fits file name 1]
	    } else {
		set fn {none}
	    }

	    # disable by default
	    $analysis(menu,$i,parent) entryconfig $analysis(menu,$i,item) \
		-state disabled

	    foreach t $analysis(menu,$i,template) {
		if {[regexp ".$t" $fn]} {
		    $analysis(menu,$i,parent) entryconfig \
			$analysis(menu,$i,item) -state normal
		    break
		}
	    }
	}
    } else {
	$ds9(mb) entryconfig $menu(analysis) -state disabled
    }
}

