#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc EditMenu {} {
    global ds9
    global menu

    set menu(edit,none) [msgcat::mc "None"]
    set menu(edit,pointer) [msgcat::mc "Pointer"]
    set menu(edit,crosshair) [msgcat::mc "Crosshair"]
    set menu(edit,colorbar) [msgcat::mc "Colorbar"]
    set menu(edit,pan) [msgcat::mc "Pan"]
    set menu(edit,zoom) [msgcat::mc "Zoom"]
    set menu(edit,rotate) [msgcat::mc "Rotate"]
    set menu(edit,catalog) [msgcat::mc "Catalog"]
    set menu(edit,examine) [msgcat::mc "Examine"]

    menu $ds9(mb).edit -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor) -title "DS9 Edit"
    $ds9(mb).edit add command -label $menu(edit,undo) -command UndoFrame
    $ds9(mb).edit add separator
    $ds9(mb).edit add command -label $menu(edit,cut) -command CutFrame
    $ds9(mb).edit add command -label $menu(edit,copy) -command CopyFrame
    $ds9(mb).edit add command -label $menu(edit,paste) -command PasteFrame
    $ds9(mb).edit add separator
    $ds9(mb).edit add radiobutton -label $menu(edit,none) \
	-variable ds9(mode) -value none -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,pointer) \
	-variable ds9(mode) -value pointer -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,crosshair) \
	-variable ds9(mode) -value crosshair -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,colorbar) \
	-variable ds9(mode) -value colorbar -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,pan) \
	-variable ds9(mode) -value pan -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,zoom) \
	-variable ds9(mode) -value zoom -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,rotate) \
	-variable ds9(mode) -value rotate -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,catalog) \
	-variable ds9(mode) -value catalog -command ChangeMode
    $ds9(mb).edit add radiobutton -label $menu(edit,examine) \
	-variable ds9(mode) -value examine -command ChangeMode
    $ds9(mb).edit add separator
    $ds9(mb).edit add cascade -label $menu(preferences) \
	-menu $ds9(mb).prefs

    EditPrefsMenu
}

proc EditPrefsMenu {} {
    global ds9
    global menu
    global prefs

    menu $ds9(mb).prefs.gui.edit -tearoff $menu(tearoff) \
	-selectcolor $menu(selectcolor)
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,none) \
	-variable prefs(ds9,mode) -value none -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,pointer) \
	-variable prefs(ds9,mode) -value pointer -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,crosshair) \
	-variable prefs(ds9,mode) -value crosshair -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,colorbar) \
	-variable prefs(ds9,mode) -value colorbar -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,pan) \
	-variable prefs(ds9,mode) -value pan -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,zoom) \
	-variable prefs(ds9,mode) -value zoom -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,rotate) \
	-variable prefs(ds9,mode) -value rotate -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,catalog) \
	-variable prefs(ds9,mode) -value catalog -command SavePrefs
    $ds9(mb).prefs.gui.edit add radiobutton -label $menu(edit,examine) \
	-variable prefs(ds9,mode) -value examine -command SavePrefs
}

proc EditButtons {} {
    global ds9
    global buttons

    set buttons(edit,undo) [msgcat::mc "undo"]
    set buttons(edit,cut) [msgcat::mc "cut"]
    set buttons(edit,copy) [msgcat::mc "copy"]
    set buttons(edit,paste) [msgcat::mc "paste"]
    set buttons(edit,none) [msgcat::mc "none"]
    set buttons(edit,pointer) [msgcat::mc "pointer"]
    set buttons(edit,crosshair) [msgcat::mc "crosshair"]
    set buttons(edit,colorbar) [msgcat::mc "colorbar"]
    set buttons(edit,pan) [msgcat::mc "pan"]
    set buttons(edit,zoom) [msgcat::mc "zoom"]
    set buttons(edit,rotate) [msgcat::mc "rotate"]
    set buttons(edit,catalog) [msgcat::mc "catalog"]
    set buttons(edit,examine) [msgcat::mc "examine"]

    frame $ds9(buttons).edit
    ButtonButton $ds9(buttons).edit.undo $buttons(edit,undo) UndoFrame
    ButtonButton $ds9(buttons).edit.cut $buttons(edit,cut) CutFrame
    ButtonButton $ds9(buttons).edit.copy $buttons(edit,copy) CopyFrame
    ButtonButton $ds9(buttons).edit.paste $buttons(edit,paste) PasteFrame
    RadioButton $ds9(buttons).edit.default $buttons(edit,none) \
	ds9(mode) none ChangeMode
    RadioButton $ds9(buttons).edit.pointer $buttons(edit,pointer) \
	ds9(mode) pointer ChangeMode
    RadioButton $ds9(buttons).edit.crosshair $buttons(edit,crosshair) \
	ds9(mode) crosshair ChangeMode
    RadioButton $ds9(buttons).edit.colorbar $buttons(edit,colorbar) \
	ds9(mode) colorbar ChangeMode
    RadioButton $ds9(buttons).edit.pan $buttons(edit,pan) \
	ds9(mode) pan ChangeMode
    RadioButton $ds9(buttons).edit.zoom $buttons(edit,zoom) \
	ds9(mode) zoom ChangeMode
    RadioButton $ds9(buttons).edit.rotate $buttons(edit,rotate) \
	ds9(mode) rotate ChangeMode
    RadioButton $ds9(buttons).edit.catalog $buttons(edit,catalog) \
	ds9(mode) catalog ChangeMode
    RadioButton $ds9(buttons).edit.examine $buttons(edit,examine) \
	ds9(mode) examine ChangeMode
}

proc UpdateEditMenu {} {
    global ds9
    global current
    global marker
    global menu

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateEditMenu"
    }

    switch -- $ds9(mode) {
	pointer {
	    if {$current(frame) != {}} {
		set l [$current(frame) has marker undo]
		if {$l != {}} {
		    $ds9(mb).edit entryconfig 1 \
			-label "$menu(edit,undo) [string totitle $l]" \
			-state normal

		    $ds9(buttons).edit.undo configure -state normal
		} else {
		    $ds9(mb).edit entryconfig 1 -label $menu(edit,undo) \
			-state disabled
		    $ds9(buttons).edit.undo configure -state disabled
		}

		if {[$current(frame) has marker select]} {
		    $ds9(mb).edit entryconfig $menu(edit,cut) -state normal
		    $ds9(mb).edit entryconfig $menu(edit,copy) -state normal
		    $ds9(buttons).edit.cut configure -state normal
		    $ds9(buttons).edit.copy configure -state normal
		} else {
		    $ds9(mb).edit entryconfig $menu(edit,cut) -state disabled
		    $ds9(mb).edit entryconfig $menu(edit,copy) -state disabled
		    $ds9(buttons).edit.cut configure -state disabled
		    $ds9(buttons).edit.copy configure -state disabled
		}

		if {$marker(copy) != {} } {
		    if {[$marker(copy) has marker paste]} {
			$ds9(mb).edit entryconfig $menu(edit,paste) \
			    -state normal
			$ds9(buttons).edit.paste configure -state normal
		    } else {
			$ds9(mb).edit entryconfig $menu(edit,paste) \
			    -state disabled
			$ds9(buttons).edit.paste configure -state disabled
		    }
		} else {
		    $ds9(mb).edit entryconfig $menu(edit,paste) -state disabled
		    $ds9(buttons).edit.paste configure -state disabled
		}
	    } else {
		$ds9(mb).edit entryconfig 1 -label $menu(edit,undo) \
		    -state disabled
		$ds9(mb).edit entryconfig $menu(edit,cut) -state disabled
		$ds9(mb).edit entryconfig $menu(edit,copy) -state disabled
		$ds9(mb).edit entryconfig $menu(edit,paste) -state disabled

		$ds9(buttons).edit.undo configure -state disabled
		$ds9(buttons).edit.cut configure -state disabled
		$ds9(buttons).edit.copy configure -state disabled
		$ds9(buttons).edit.paste configure -state disabled
	    }
	}
	none -
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	catalog -
	imexam {$ds9(mb).edit entryconfig 1 -label $menu(edit,undo) \
		    -state disabled}
    }
}

