#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc NRESApply {varname sync} {
    upvar #0 $varname var
    global $varname

    set var(sync) $sync

    ARApply $varname

    if {$var(name)!={}} {
	NSVRServer $varname
    } else {
	ARStatus $varname "Please specify an Object Name"
	ARReset $varname
    }
}

proc NRESDialog {} {
    global ds9
    global menu
    global nres

    if [winfo exists $nres(top)] {
	raise $nres(top)
	return
    }

    set varname nresi
    upvar #0 $varname var
    global $varname

    # AR variables
    ARInit $varname {}

    # Variables
    set var(top) $nres(top)
    set var(mb) $nres(mb)
    set var(server) $nres(server)
    set var(system) $nres(system)
    set var(sky) $nres(sky)
    set var(skyformat) $nres(skyformat)

    # top
    set w $var(top)
    set title "Name Resolution"

    # create the window
    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "ARDestroy $varname"
    wm minsize $w 375 100

    # menu
    $w configure -menu $var(mb)
    menu $var(mb) -tearoff 0

    # file
    $var(mb) add cascade -label File -menu $var(mb).file
    menu $var(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).file add command -label $menu(file,retrieve) \
	-command "NRESApply $varname 0"
    $var(mb).file add command -label $menu(file,cancel) \
	-command "ARCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "Pan To" -command "NRESPan $varname"
    $var(mb).file add command -label "Crosshair To" \
	-command "NRESCrosshair $varname"
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,close) \
	-command "ARDestroy $varname"

    # edit
    AREditMenu $varname

    # name server
    NSVRServerMenu $varname

    # name format
    $var(mb) add cascade -label Format -menu $var(mb).format
    menu $var(mb).format -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).format add radiobutton -label $menu(wcs,degrees) \
	-variable ${varname}(skyformat) -value degrees
    $var(mb).format add radiobutton -label $menu(wcs,sexagesimal) \
	-variable ${varname}(skyformat) -value sexagesimal

    # dialog
    frame $w.param -relief groove -borderwidth 2
    frame $w.status -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.status $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    label $w.param.nametitle -text Name
    entry $w.param.name -textvariable ${varname}(name) -width 32

    set var(xname) [label $w.param.xtitle -text "" -font {symbol 12} -width 1]
    label $w.param.x -textvariable ${varname}(x) -width 14 -relief groove
    set var(yname) [label $w.param.ytitle -text "" -font {symbol 12} -width 1]
    label $w.param.y -textvariable ${varname}(y) -width 14 -relief groove
    label $w.param.system -textvariable ${varname}(sky) \
	-width 10 -relief groove

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4

    grid $w.param.nametitle $w.param.name - - -padx 4 -pady 1 -sticky w
    grid $w.param.xtitle $w.param.x $w.param.ytitle $w.param.y \
	$w.param.system -padx 4 -pady 1 -sticky w

    label $w.status.title -text "Status:"
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.title $w.status.item -side left -anchor w -pady 4

    set var(apply) [button $w.buttons.apply -text $menu(file,retrieve) \
			-command "NRESApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text $menu(file,cancel) \
			 -command "ARCancel  $varname" -state disabled]

    button $w.buttons.close -text $menu(file,close) \
	-command "ARDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.close \
	-side left -padx 10 -expand true

    ARCoord $varname
    ARStatus $varname {}
}

proc NRESPan {varname} {
    upvar #0 $varname var
    global $varname

    if {($var(x) != {}) && ($var(y) != {})} {
	PanTo $var(x) $var(y) $var(system) $var(sky)
    }
}

proc NRESCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    global current
    global ds9

    if {($current(frame) != {})} {
	if {[$current(frame) has wcs equatorial $var(system)]} {
	    if {($var(x) != {}) && ($var(y) != {})} {
		set ds9(mode) crosshair
		ChangeMode
		$current(frame) crosshair \
		    $var(system) $var(sky) $var(x) $var(y)
	    }
	}
    }
}

# Process Cmds

proc ProcessNRESCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    set vvarname nresi
    upvar #0 $vvarname vvar
    global $vvarname

    NRESDialog

    switch -- [string tolower [lindex $var $i]] {
	server {
	    incr i
	    set vvar(server) [lindex $var $i]
	}
	format -
	skyformat {
	    incr i
	    set vvar(skyformat) [lindex $var $i]
	}
	name {
	    incr i
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
	default {
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
    }
}
