#  Copyright (C) 1999-2005
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc VOCancel {varname} {
    upvar #0 $varname var
    global $varname

    # set state to 0 so that we don't process the finish proc
    set var(active) 0

    if [info exists var(token)] {
	http::reset $var(token)
    }
}

proc VODestroy {varname} {
    upvar #0 $varname var
    global $varname

    VOCancel $varname

    if {[winfo exists $var(top)]} {
	destroy $var(top)
	destroy $var(mb)
    }

    unset $varname
}

proc VOReset {varname} {
    upvar #0 $varname var
    global $varname

    set var(active) 0

    if {[info exists var(token)]} {
	http::cleanup $var(token)
	unset var(token)
    }
}

proc VODone {varname} {
    upvar #0 $varname var
    global $varname

    VOReset $varname
}

proc VOCancelled {varname} {
    upvar #0 $varname var
    global $varname

    VOReset $varname
}

proc VOError {varname message} {
    upvar #0 $varname var
    global $varname

    Error $message
    VOReset $varname
}

proc VODialog {sync} {
    global ds9
    global vo
    global menu
    global current

    if [winfo exists $vo(top)] {
	raise $vo(top)
	return
    }

    set varname voi
    upvar #0 $varname var
    global $varname

    # variables
    set var(top) $vo(top)
    set var(mb) $vo(mb)
    set var(sync) $sync

    set w $var(top)
    set title "Virtual Observatory"

    # create the window
    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "VODestroy $varname"

    # menu
    $w configure -menu $var(mb)
    menu $var(mb) -tearoff 0
    $var(mb) add cascade -label $menu(file) -menu $var(mb).file
    $var(mb) add cascade -label "Configure" -menu $var(mb).config
    $var(mb) add cascade -label $menu(help) -menu $var(mb).help

    menu $var(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).file add command -label $menu(file,cancel) \
	-command "VOCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label $menu(file,close) \
	-command "VODestroy $varname"

    menu $var(mb).config -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).config add checkbutton -label "Internal Browser" \
	-variable vo(hv) -command SavePrefs
    $var(mb).config add separator
    $var(mb).config add radiobutton -label "Connect Direct" \
	-variable vo(method) -value xpa -command PrefsVOMethod
    $var(mb).config add radiobutton -label "Connect Web Proxy" \
	-variable vo(method) -value mime -command PrefsVOMethod
    $var(mb).config add separator
    $var(mb).config add command -label "Configure Web Proxy" -command PrefsHTTP
    $var(mb).config add command -label "Select VO Server" \
	-command "VOConfigServer $varname"

    menu $var(mb).help -tearoff 0 -selectcolor $menu(selectcolor)
    $var(mb).help add command -label "Help me Choose" -command HelpVO

    frame $w.param -relief groove -borderwidth 2
    frame $w.opt -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.opt $w.buttons \
	-fill x -expand true -ipadx 4 -ipady 4

    frame $w.opt.o
    pack $w.opt.o -side left
    label $w.opt.o.t1  -text "Use Internal Web Browser:"
    set var(hv,button) [checkbutton $w.opt.o.hv -text "" \
			    -variable vo(hv) \
			    -selectcolor $menu(selectcolor) \
			    -command SavePrefs]
    label $w.opt.o.t2  -text "Connect to VO Servers:"
    radiobutton $w.opt.o.xpa -text "Directly" -variable vo(method) \
	-value xpa -command PrefsVOMethod \
	-selectcolor $menu(selectcolor)
    radiobutton $w.opt.o.http -text "Using Web Proxy" \
	-variable vo(method) \
	-value mime -command PrefsVOMethod \
	-selectcolor $menu(selectcolor)
    button $w.opt.o.help -text "Help Me Choose" -command HelpVO
    button $w.opt.o.proxy -text "Configure Web Proxy" -command PrefsHTTP

    grid $w.opt.o.t1 $w.opt.o.hv -padx 4 -sticky w
    grid $w.opt.o.t2 $w.opt.o.xpa $w.opt.o.http -padx 4 -sticky w
    grid x $w.opt.o.help $w.opt.o.proxy -padx 4 -sticky w

    button $w.buttons.close -text $menu(file,close) \
	-command "VODestroy $varname"
    pack $w.buttons.close -side left -padx 10 -expand true

    if {[string length $vo(server,host)] == 0} {
	VOApply $varname
    } else {
	set l [llength $vo(server,host)]
	for {set ii 0} {$ii<$l} {incr ii} {
	    set b [lindex $vo(server,button) $ii]
	    checkbutton $b -text "[lindex $vo(server,title) $ii] ([lindex $vo(server,url) $ii])" -variable vo(b$ii) -command "VOCheck $varname $ii"
	    pack $b -anchor w -padx 4

	    # update color
	    if {$vo(b$ii)} {
		$b configure -selectcolor yellow

		global debug
		if {$debug(tcl,idletasks)} {
		    puts "VODialog"
		}
		update idletasks

		$b configure -selectcolor green
	    } else {
		$b configure -selectcolor yellow
		$b select

		global debug
		if {$debug(tcl,idletasks)} {
		    puts "VODialog"
		}
		update idletasks

		$b configure -selectcolor {}
		$b deselect
	    }
	}
    }
}

proc VOApply {varname} {
    upvar #0 $varname var
    global $varname
    global vo

    global xpa
    global http

    set w $var(top)

    # delete old servers
    for {set ii 0} {$ii < [llength $vo(server,button)]} {incr ii} {
	catch {xparemote $xpa [lindex $vo(server,host) $ii] - -proxy}
	catch {destroy [lindex $vo(server,button) $ii]}
	catch {unset vo(b$ii)}
    }
    set vo(server,host) {}
    set vo(server,title) {}
    set vo(server,url) {}
    set vo(server,button) {}

    set var(url) $vo(server)
    VOLoad $varname 
}

proc VOLoad {varname} {
    upvar #0 $varname var
    global $varname

    global http

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -timeout $http(timeout) \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    VOFinish $varname $var(token)
	} else {
	    VOError $varname "Unable to locate URL $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -timeout $http(timeout) \
					 -command [list VOFinish $varname] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    VOError $varname "Unable to locate URL $var(url)"
	}
    }
}

proc VOFinish {varname token} {
    upvar #0 $varname var
    global $varname

    if {!($var(active))} {
	VOCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	200 -
	203 -
	503 {
	    VOParse $varname
	    VODone $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    if {$debug(tcl,hv)} {
			puts "VOFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    VOLoad $varname
		}
	    }
	}

	default {VOError $varname \
		     "An error has occured while updating VO server list."}
    }
}

proc VOParse {varname} {
    upvar #0 $varname var
    global $varname
    global vo

    set w $var(top)

    set data [string trimright [http::data $var(token)] \n]

    set lines [split $data \n]
    set l [llength $lines]
    for {set ii 0} {$ii<$l} {incr ii} {
	set line [lindex $lines $ii]
	set b "$w.param.b$ii"

	set ll [split $line \t]
	lappend vo(server,host) [lindex $ll 0]
	lappend vo(server,title) [lindex $ll 1]
	lappend vo(server,url) [lindex $ll 2]
	lappend vo(server,button) $b
	checkbutton $b -text "[lindex $vo(server,title) $ii] ([lindex $vo(server,url) $ii])" -variable vo(b$ii) -command "VOCheck $varname $ii"
	pack $b -anchor w -padx 4
    }
}

proc VOCheck {varname ii} {
    upvar #0 $varname var
    global $varname
    global vo

    global xpa

    set w $var(top)
    set b "$w.param.b$ii"

    if {$vo(b$ii)} {
	$b configure -selectcolor yellow

	global debug
	if {$debug(tcl,idletasks)} {
	    puts "VOCheck"
	}
	update idletasks

	switch $vo(method) {
	    mime {}
	    xpa {
		if {[catch {xparemote $xpa \
				[lindex $vo(server,host) $ii] + -proxy}]} {
		    $b configure -selectcolor {}
		    $b deselect
		    set vo(b$ii) 0
		    Error "Unable to connect to VO server"
		    return
		}
	    }
	}

	$b configure -selectcolor green

	if {$vo(hv)} {
	    set url [lindex $vo(server,url) $ii]
	    ParseURL $url r
	    HV "vo$ii" "$r(authority)" $url 4 1 {}
	}
    } else {
	$b configure -selectcolor yellow
	$b select

	global debug
	if {$debug(tcl,idletasks)} {
	    puts "VOCheck"
	}
	update idletasks

	switch $vo(method) {
	    mime {}
	    xpa {
		catch {xparemote $xpa [lindex $var(server,host) $ii] - -proxy}
	    }
	}

	$b configure -selectcolor {}
	$b deselect
    }
}

proc VOConfigServer {varname} {
    upvar #0 $varname var
    global $varname

    if [PrefsVOServer] {
	VOApply $varname
    }
}

# Prefs Cmds

proc PrefsVOMethod {} {
    global vo

    switch $vo(method) {
	mime {set vo(hv) 1}
	xpa {}
    }
    SavePrefs
}

proc PrefsVOServer {} {
    global vo

    set result $vo(server)
    if [EntryDialog "VO Server" "Enter VO Server List URL:" 80 result] {
	set vo(server) $result
	SavePrefs
	return 1
    }
    return 0
}

# Process Cmds

proc ProcessVOCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global vo

    set vvarname voi
    upvar #0 $vvarname vvar
    global $vvarname

    VODialog 1

    # find best match
    set ii [lsearch $vo(server,url) "*[lindex $var $i]*"]
    if {$ii>=0} {
	set vo(b$ii) 1
	VOCheck $vvarname $ii
    }
}
